# pylistmanager
# Copyright (C) 2018  Sotiris Papatheodorou  (sotirisp@protonmail.com)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# -*- coding: utf-8 -*-


import argparse
import os
import sys
from pathlib import PurePath
from xdg import XDG_DATA_HOME
import colorama
from .__version__ import __version__
from .movie import *
from .common import *


# Global variables
modes = ['list', 'add', 'delete', 'remove', 'edit', 'convert', 'tags']


def info():
    print(str(PurePath(sys.argv[0]).name) + ' ' + str(__version__))
    print("""Copyright (C) 2018  Sotiris Papatheodorou
License GPLv3: GNU GPL version 3 <https://gnu.org/licenses/gpl.html>.
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.""")


def cli_arg_parser():
    parser = argparse.ArgumentParser(
        description='Movie list manager',
        formatter_class=argparse.RawTextHelpFormatter
    )
    parser.add_argument(
        'mode', choices=modes+[x[0] for x in modes], nargs='?', default='list',
        help='l, list     list all movies in the file\n\
a, add      add a single movie to the file\n\
d, delete   delete a single movie from the file\n\
r, remove   alias for d, delete\n\
e, edit     edit a single movie from the file\n\
c, convert  convert the file to the native csv format in-place\n\
t, tags     show only the movies from the file matching some tags\n\
'
    )
    parser.add_argument(
        '-f', '--file', nargs='?', 
        default=XDG_DATA_HOME + '/pymovielist/movies.csv',
        help='the csv file containing the movie list'
    )
    parser.add_argument(
        '--version', const=True, default=False, action='store_const',
        help='show the version and license information and exit'
    )
    args = parser.parse_args()
    args.file = args.file.strip()
    args.file = os.path.expanduser(args.file)
    return args


def main():
    colorama.init()

    # Parse command line arguments
    args = cli_arg_parser()
    if args.version:
        info()
        return

    # Perform action depending on selected mode
    if args.mode == 'list' or args.mode == 'l':
        # list
        lst = Movies()
        try:
            lst.read(args.file)
        except FileNotFoundError:
            print('File \'' + args.file + '\' could not be opened.')
            print('You can create the file and add a movie to it by running:')
            if args.file == XDG_DATA_HOME + '/pymovielist/movies.csv':
                print('    movielist add')
            else:
                print('    movielist -f \"' + args.file + '\" add')
            return
        print(lst)
    elif args.mode == 'add' or args.mode == 'a':
        # add
        m = Movie()
        m.read_from_user()
        m.write(args.file)
        print('\nAdded:')
        print(m)
        print('to: ' + args.file)
    elif (args.mode == 'remove' or args.mode == 'r'
            or args.mode == 'delete' or args.mode == 'd'):
        # remove
        lst = Movies()
        try:
            lst.read(args.file)
        except FileNotFoundError:
            print('File \'' + args.file + '\' could not be opened.')
            print('You can create the file and add a movie to it by running:')
            if args.file == XDG_DATA_HOME + '/pymovielist/movies.csv':
                print('    movielist add')
            else:
                print('    movielist -f \"' + args.file + '\" add')
            return
        print(lst)
        # Movie selection
        selection = input_int('Select movie to delete: ', 1, len(lst.list))
        selection = selection - 1
        print('\nSelected movie details\n')
        print(lst.list[selection])
        # Delete after confirmation
        confirm = input_loop(process_yn_input,
                             'Delete movie and Write changes to \''
                             + args.file + '\'? [y/n] ')
        if confirm:
            del lst.list[selection]
            lst.write(args.file, 'w')
            print('Changes written successfully')
    elif args.mode == 'edit' or args.mode == 'e':
        # edit
        lst = Movies()
        try:
            lst.read(args.file)
        except FileNotFoundError:
            print('File \'' + args.file + '\' could not be opened.')
            print('You can create the file and add a movie to it by running:')
            if args.file == XDG_DATA_HOME + '/pymovielist/movies.csv':
                print('    movielist add')
            else:
                print('    movielist -f \"' + args.file + '\" add')
            return
        print(lst)
        # Movie selection
        selection = input_int('Select movie to edit: ', 1, len(lst.list))
        selection = selection - 1
        # Edit movie data
        lst.list[selection].edit_from_user()
        print('\nChanged movie details\n')
        print(lst.list[selection])
        # Write movie list after confirmation
        confirm = input_loop(process_yn_input, 'Write changes to \''
                             + args.file + '\'? [y/n] ')
        if confirm:
            lst.write(args.file, 'w')
            print('Changes written successfully')
    elif args.mode == 'convert' or args.mode == 'c':
        # convert
        lst = Movies()
        try:
            lst.import_data(args.file)
        except FileNotFoundError:
            print('File \'' + args.file + '\' could not be opened.')
            return
        if len(lst.list) > 0:
            confirm = input_loop(process_yn_input, 'Write changes to \''
                                 + args.file
                                 + '\'? Old contents will be lost. [y/n] ')
            if confirm:
                lst.write(args.file, 'w')
                print('Converted ' + str(len(lst.list))
                      + ' movies from \'' + args.file + '\'')
    elif args.mode == 'tags' or args.mode == 't':
        # tags
        lst = Movies()
        try:
            lst.read(args.file)
        except FileNotFoundError:
            print('File \'' + args.file + '\' could not be opened.')
            print('You can create the file and add a movie to it by running:')
            if args.file == XDG_DATA_HOME + '/pymovielist/movies.csv':
                print('    movielist add')
            else:
                print('    movielist -f \"' + args.file + '\" add')
            return
        # Tag selection
        s = input('Enter tags to show (comma separated): ')
        s = s.rstrip(' ,')
        tags = [x.strip() for x in s.split(',')]
        tags = list(dict.fromkeys(tags))
        # Iterate over movie list and print if any tags match
        for movie in lst.list:
            for movie_tag in movie.tags:
                if movie_tag in tags:
                    tmp_str = str(movie)
                    for movie_tag2 in movie.tags:
                        if movie_tag2 in tags:
                            tmp_str = tmp_str.replace(movie_tag2,
                                style_search_results + movie_tag2
                                + Style.RESET_ALL)
                    print(tmp_str)
                    break




# pylistmanager
# Copyright (C) 2018  Sotiris Papatheodorou  (sotirisp@protonmail.com)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# -*- coding: utf-8 -*-


import datetime


class ListItem:
    def __init__(self):
        self.id = 1
        self.name = 'NO NAME'
        self.date_added = datetime.date.today()
        self.tags = []

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def tags_from_str(self, s):
        """Given a string s containing comma-separated tags, place one tag in
        each element of the tag attribute list.
        """
        s = s.rstrip(' ,')
        self.tags = [x.strip() for x in s.split(',')]
        # Remove duplicate keys
        self.tags = list(dict.fromkeys(self.tags))
        self.tags.sort()


class ListItems:
    def __init__(self):
        self.list = []




# pylistmanager
# Copyright (C) 2018  Sotiris Papatheodorou  (sotirisp@protonmail.com)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# -*- coding: utf-8 -*-


import datetime


def date_from_str(s):
    """Convert a string to a date object.
    
    Args:
        s (str): A string containing a date in the format YYYY-MM-DD.

    Returns:
        date: A datetime.date object.

    Raises:
        ValueError: If the string is in the wrong format or if the date given
        is invalid.
    """

    dt = s.split('-')
    if len(dt) != 3:
        raise ValueError()
    return datetime.date(int(dt[0]), int(dt[1]), int(dt[2]))

def input_loop(process_input, prompt=''):
    while True:
        try:
            s = input(prompt)
            result = process_input(s)
            break
        except ValueError:
            pass
    return result

def input_int(prompt='', min_value=0, max_value=1):
    while True:
        try:
            i = int(input(prompt))
            if i < min_value or i > max_value:
                raise ValueError
            break
        except ValueError:
            print('Enter an integer between ' + str(min_value)
                  + ' and ' + str(max_value))
    return i

def process_yn_input(s):
    yes_answers = ['y', 'Y', 'yes', 'Yes', 'YES']
    no_answers = ['n', 'N', 'no', 'No', 'NO']
    if s in yes_answers:
        return True
    elif s in no_answers:
        return False
    else:
        print('Enter y or n')
        raise ValueError

def process_rating_input(s):
    if s:
        # Get rating and max_rating from a string in the format
        # rating/max_rating
        sub_s = s.split('/')
        rating = float(sub_s[0])
        if len(sub_s) > 1:
           max_rating = float(sub_s[1])
        else:
           max_rating = 10.0
        # Test if rating is inside the bounds
        if rating < 0 or rating > max_rating or max_rating <= 0:
            print('Rating must be a number in the interval [0, '
                  + str(max_rating) + ']')
            raise ValueError
    else:
       rating = -1
       max_rating = 10.0
    return [rating, max_rating]

def process_year_input(s):
    try:
        y = int(s)
        if (y < datetime.MINYEAR or y > datetime.MAXYEAR):
            raise ValueError()
    except ValueError:
        print('Value must be an integer between '
              + str(datetime.MINYEAR) + ' and ' + str(datetime.MAXYEAR))
        raise ValueError
    return y

def process_date_input(s):
    try:
        if s in ['t', 'today', 'T', 'TODAY']:
            d = datetime.date.today()
        elif s in ['y', 'yesterday', 'Y', 'YESTERDAY']:
            d = datetime.date.today() - datetime.timedelta(days=1)
        elif s in ['n', 'never', 'N', 'NEVER'] or not s:
            d = ''
        else:
            d = date_from_str(s)
    except ValueError:
        print('Enter date in YYYY-MM-DD format')
        raise ValueError
    return d




# pylistmanager
# Copyright (C) 2018  Sotiris Papatheodorou  (sotirisp@protonmail.com)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# -*- coding: utf-8 -*-


import csv
import datetime
import os
import sys
from colorama import Fore, Back, Style
from .listitem import *
from .common import *


# Global variables
movie_attributes = ['name', 'year', 'rating', 'max_rating', 'tags', 
                    'date_watched', 'imdb_URI', 'rtomatoes_URI',
                    'letterboxd_URI', 'date_added']
csv_header_columns = ['Name', 'Year Released', 'Rating', 'Max Rating', 'Tags',
                      'Date Watched', 'IMDB URI', 'Rotten Tomatoes URI',
                      'Letterboxd URI', 'Date Added']
# Generate header-attribute dictionaries and native csv header
csv_to_attr = dict(zip(csv_header_columns, movie_attributes))
attr_to_csv = dict(zip(movie_attributes, csv_header_columns))
csv_header = ''.join([x+',' for x in csv_header_columns]).rstrip(',')
# csv headers for conversion
letterboxd_headers = [
    'Date,Name,Year,Letterboxd URI',
    'Date,Name,Year,Letterboxd URI,Rating,Tags,Watched Date'
]
# Color settings
style_title = Fore.GREEN + Style.BRIGHT
style_attributes = Style.BRIGHT
style_number = Style.BRIGHT
style_search_results = Fore.YELLOW

class Movie(ListItem):
    def __init__(self, name='Movie Name', year=datetime.date.today().year):
        super().__init__()
        self.name = name
        self.year = int(year)
        self.date_watched = datetime.date(datetime.MINYEAR, 1, 1)
        self.rating = -1
        self.max_rating = 10.0
        self.letterboxd_URI = ''
        self.imdb_URI = ''
        self.rtomatoes_URI = ''

    def change_rating_scale(self, new_max_rating=10.0):
        self.rating = self.rating / self.max_rating * new_max_rating
        self.max_rating = new_max_rating

    def __str__(self):
        # Compute the number of spaces to pad with
        nspaces = len(str(self.id)) + 2
        padding = ''.join([' ' for i in range(nspaces)])
        s = [
            style_number + str(self.id),
            ') ' + Style.RESET_ALL,
            style_title + self.name + Fore.RESET,
            '\n' + padding + style_attributes + 'Year: ' + Style.RESET_ALL,
            str(self.year),
            '\n'
        ]
        if self.rating >= 0:
            s.append(padding + style_attributes + 'Rating: '  + Style.RESET_ALL
                     + "{:.2f}".format(self.rating) + '/10\n')
        if self.tags:
            s.append(padding + style_attributes + 'Tags: ' + Style.RESET_ALL)
            [s.append(str(x)+', ') for x in self.tags]
            s.append('\b\b  \n')
        if self.date_watched != datetime.date(1,1,1):
            s.append(padding + style_attributes + 'Watched on: '
                     + Style.RESET_ALL + str(self.date_watched) + '\n')
        if self.imdb_URI:
            s.append(padding + style_attributes + 'IMDB URI: '
                     + Style.RESET_ALL + str(self.imdb_URI) + '\n')
        if self.rtomatoes_URI:
            s.append(padding + style_attributes + 'Rotten Tomatoes URI: '
                     + Style.RESET_ALL + str(self.rtomatoes_URI) + '\n')
        if self.letterboxd_URI:
            s.append(padding + style_attributes + 'Letterboxd URI: '
                     + Style.RESET_ALL + str(self.letterboxd_URI) + '\n')
        s.append(padding + style_attributes + 'Added on: ' + Style.RESET_ALL
                 + str(self.date_added) + '\n')
        return ''.join(s)

    def to_csv_str(self):
        s = []
        s.append('"' + self.name + '"' + ',')
        s.append(str(self.year) + ',')
        if self.rating >= 0:
            s.append(str(self.rating))
        s.append(',')
        s.append(str(self.max_rating) + ',')
        if self.tags and self.tags[0]:
            s.append('"')
            [s.append(str(x)+',') for x in self.tags]
            s[-1] = s[-1].rstrip(',')
            s.append('"')
        s.append(',')
        if self.date_watched != datetime.date(1,1,1):
            s.append(str(self.date_watched))
        s.append(',')
        s.append(str(self.imdb_URI) + ',')
        s.append(str(self.rtomatoes_URI) + ',')
        s.append(str(self.letterboxd_URI) + ',')
        if self.date_added != datetime.date(1,1,1):
            s.append(str(self.date_added))
        return ''.join(s)

    def read_from_user(self):
        """Read the attributes of the new movie object from standard input.
        """
        print('Enter new movie details')
        # Name
        self.name = input('Name: ')
        # Year
        self.year = input_loop(process_year_input, 'Year released: ')
        # Date watched
        self.date_watched = input_loop(process_date_input,
                                       'Date watched [unwatched]: ')
        # Rating
        rating = input_loop(process_rating_input,
                            'Rating[/Max rating] [unrated]: ')
        self.rating = rating[0]
        self.max_rating = rating[1]
        self.change_rating_scale(10.0)
        # Tags
        s = input('Tags (comma separated): ')
        self.tags_from_str(s)
        # Automatically generated
        self.imdb_URI = get_imdb_URI(self.name, self.year)
        self.rtomatoes_URI = get_rtomatoes_URI(self.name, self.year)
        self.letterboxd_URI = get_letterboxd_URI(self.name, self.year)

    def edit_from_user(self):
        """Read the attributes of the movie object from standard input.
        """
        print('Change movie details')
        # Name
        s = input('Name [' + self.name +  ']: ')
        if s:
            self.name = s
        # Year
        self.year = input_loop(process_year_input, 'Year released ['
                               + str(self.year) + ']: ')
        # Date watched
        if self.date_watched == datetime.date(datetime.MINYEAR, 1, 1):
            self.date_watched = input_loop(process_date_input,
                                           'Date watched [unwatched]: ')
        else:
            self.date_watched = input_loop(process_date_input,
                                           'Date watched ['
                                           + str(self.date_watched) + ']: ')
        # Rating
        rating = input_loop(process_rating_input, 'Rating [' + str(self.rating)
                            + '/' + str(self.max_rating) + ']: ')
        if rating[0] > 0:
            self.rating = rating[0]
            self.max_rating = rating[1]
            self.change_rating_scale(10.0)
        # Tags
        taglist = []
        [taglist.append(str(x)+', ') for x in self.tags]
        tagstr = ''.join(taglist)
        tagstr = tagstr.rstrip(', ')
        s = input('Tags [' + tagstr + ']: ')
        if s:
            self.tags_from_str(s)
        # IMDB URI
        s = input('IMDB URI [' + self.imdb_URI +  ']: ')
        if s:
            self.imdb_URI = s
        # Rotten Tomatoes URI
        s = input('Rotten Tomatoes URI [' + self.rtomatoes_URI +  ']: ')
        if s:
            self.rtomatoes_URI = s
        # Letterboxd URI
        s = input('Letterboxd URI [' + self.letterboxd_URI +  ']: ')
        if s:
            self.letterboxd_URI = s

    def write(self, fname, mode='a'):
        if (mode == 'w'):
            write_header = True
        else:
            # Only allow 'w' and 'a' file modes
            mode = 'a'
            # Test if file is empty
            try:
                with open(os.path.expanduser(fname), 'r') as f:
                    first_char = f.read(1)
                    if first_char:
                        write_header = False
                    else:
                        write_header = True
            except FileNotFoundError:
                write_header = True

        # Write file
        try:
            with open(os.path.expanduser(fname), mode) as f:
                if write_header:
                    f.write(csv_header + '\n')
                f.write(self.to_csv_str() + '\n')
        except FileNotFoundError:
            print('Could not open file ' + str(fname) + ' for writing')


class Movies(ListItems):
    def __str__(self):
        s = [str(x)+'\n' for x in self.list]
        return ''.join(s)

    def read(self, fname):
        with open(fname, newline='') as f:
            reader = csv.reader(f)
            try:
                # Skip first line
                header = next(reader)
                for row in reader:
                    
                    rating = row[header.index(attr_to_csv['rating'])]
                    max_rating = row[header.index(attr_to_csv['max_rating'])]
                    tags = row[header.index(attr_to_csv['tags'])]
                    date_watched \
                        = row[header.index(attr_to_csv['date_watched'])]
                    imdb_URI = row[header.index(attr_to_csv['imdb_URI'])]
                    rtomatoes_URI \
                        = row[header.index(attr_to_csv['rtomatoes_URI'])]
                    letterboxd_URI \
                        = row[header.index(attr_to_csv['letterboxd_URI'])]
                    if len(row) == len(attr_to_csv):
                        date_added \
                            = row[header.index(attr_to_csv['date_added'])]
                    else:
                        date_added = ''

                    self.list.append(Movie())
                    # ID
                    self.list[-1].id = len(self.list)
                    # Name
                    self.list[-1].name = row[header.index(attr_to_csv['name'])]
                    # Year
                    self.list[-1].year \
                        = int(row[header.index(attr_to_csv['year'])])
                    # Rating
                    if rating:
                        self.list[-1].rating = float(rating)
                    # Max rating
                    if max_rating:
                        self.list[-1].max_rating = float(max_rating)
                    # Tags
                    if tags:
                        # Split tag string to individual tags
                        self.list[-1].tags_from_str(tags)
                    # Date watched
                    if date_watched:
                        dt = date_watched.split('-')
                        self.list[-1].date_watched = datetime.date(
                            int(dt[0]),
                            int(dt[1]),
                            int(dt[2])
                        )
                    else:
                        # Otherwise set the minimum available date
                        self.list[-1].date_watched = datetime.date(
                            datetime.MINYEAR,
                            1,
                            1
                        )
                    # IMDB URI
                    if imdb_URI:
                        self.list[-1].imdb_URI = imdb_URI
                    else:
                        self.list[-1].imdb_URI \
                          = get_imdb_URI(self.list[-1].name,
                                         self.list[-1].year)
                    # Rotten Tomatoes URI
                    if rtomatoes_URI:
                        self.list[-1].rtomatoes_URI = rtomatoes_URI
                    else:
                        self.list[-1].rtomatoes_URI \
                          = get_rtomatoes_URI(self.list[-1].name,
                                              self.list[-1].year)
                    # Letterboxd URI
                    if letterboxd_URI:
                        self.list[-1].letterboxd_URI = letterboxd_URI
                    else:
                        self.list[-1].letterboxd_URI \
                          = get_letterboxd_URI(self.list[-1].name,
                                               self.list[-1].year)
                    # Date added
                    if date_added:
                        dt = date_added.split('-')
                        self.list[-1].date_added = datetime.date(
                            int(dt[0]),
                            int(dt[1]),
                            int(dt[2])
                        )

                    # Change rating scale
                    self.list[-1].change_rating_scale(10.0)
            except csv.Error as e:
                sys.exit('file {}, line {}: {}'.format(fname,
                                                       reader.line_num, e))

    def write(self, fname, mode='a'):
        if (mode == 'w'):
            write_header = True
        else:
            # Only allow 'w' and 'a' file modes
            mode = 'a'
            # Test if file is empty
            try:
                with open(os.path.expanduser(fname), 'r') as f:
                    first_char = f.read(1)
                    if first_char:
                        write_header = False
                    else:
                        write_header = True
            except FileNotFoundError:
                write_header = True

        # Write file
        try:
            with open(os.path.expanduser(fname), mode) as f:
                if write_header:
                    f.write(csv_header + '\n')
                [f.write(item.to_csv_str() + '\n') for item in self.list]
        except FileNotFoundError:
            print('Could not open file ' + str(fname) + ' for writing')

    def import_data(self, fname):
        with open(fname, newline='') as f:
            reader = csv.reader(f)
            try:
                # Detect file origin from the header
                header = next(f).rstrip('\r\n')
                if (header == csv_header):
                    print('File already in correct format, conversion aborted')
                elif (header == letterboxd_headers[0]
                    or header == letterboxd_headers[1]):
                    # Letterboxd
                    for row in reader:
                        # Name and year
                        self.list.append(Movie(row[1], row[2]))
                        # ID
                        self.list[-1].id = len(self.list)
                        # Date added
                        dt = row[0].split('-')
                        self.list[-1].date_added = datetime.date(
                            int(dt[0]),
                            int(dt[1]),
                            int(dt[2])
                        )
                        # Rating
                        if len(row) >= 5 and row[4]:
                            self.list[-1].rating = float(row[4])
                        # Tags
                        if len(row) >= 6 and row[5]:
                            # Split tag string to individual tags
                            self.list[-1].tags_from_str(row[5])
                        # Date watched if available
                        if len(row) >= 7 and row[6]:
                            dt = row[6].split('-')
                            self.list[-1].date_watched = datetime.date(
                                int(dt[0]),
                                int(dt[1]),
                                int(dt[2])
                            )
                        else:
                            # Otherwise set the minimum available year
                            self.list[-1].date_watched = datetime.date(
                                datetime.MINYEAR,
                                1,
                                1
                            )
                        # Letterboxd URI
                        if row[3]:
                            self.list[-1].letterboxd_URI = row[3]
                        else:
                            self.list[-1].letterboxd_URI \
                              = get_letterboxd_URI(self.list[-1].name,
                                                   self.list[-1].year)
                        # Rotten Tomatoes URI
                        self.list[-1].rtomatoes_URI \
                          = get_rtomatoes_URI(self.list[-1].name,
                                              self.list[-1].year)
                        # IMDB URI
                        self.list[-1].imdb_URI \
                          = get_imdb_URI(self.list[-1].name,
                                         self.list[-1].year)

                        # Change rating scale
                        self.list[-1].change_rating_scale(10.0)
                else:
                    print('Unknown csv header, conversion aborted')
            except csv.Error as e:
                sys.exit('file {}, line {}: {}'.format(fname,
                                                       reader.line_num, e))


def get_letterboxd_URI(name, year):
    """Given a string s containing a movie name, return a string containing the
    Letterboxd URI that corresponds to that movie.
    """
    # Keep only letters, numbers and spaces
    name_letterboxd = ''.join([c for c in name if c.isalnum() or c.isspace()])
    # Make all letters lower case
    name_letterboxd = name_letterboxd.lower()
    # Replace spaces with dashes
    name_letterboxd = name_letterboxd.replace(' ', '-')
    # Remove consecutive dashes
    name_letterboxd = name_letterboxd.replace('--', '-')
    # TODO Check whether two movies of the same name exist and select the one
    # with the correct year. Add a dash to the end of the URI followed by the
    # year. Also check for +-1 year.
    uri = 'https://letterboxd.com/film/' + name_letterboxd
    return uri


def get_rtomatoes_URI(name, year):
    """Given a string s containing a movie name, return a string containing the
    Rotten Tomatoes URI that corresponds to that movie.
    """
    # Keep only letters, numbers and spaces
    name_rtomatoes = ''.join([c for c in name if c.isalnum() or c.isspace()])
    # Make all letters lower case
    name_rtomatoes = name_rtomatoes.lower()
    # Replace spaces with underscores
    name_rtomatoes = name_rtomatoes.replace(' ', '_')
    # Remove consecutive underscores
    name_rtomatoes = name_rtomatoes.replace('__', '_')
    # TODO Check whether two movies of the same name exist and select the one
    # with the correct year. Add an underscore to the end of the URI followed
    # by the year. Also check for +-1 year.
    uri = 'https://www.rottentomatoes.com/m/' + name_rtomatoes
    return uri


def get_imdb_URI(name, year):
    # TODO
    return ''





# pylistmanager
# Copyright (C) 2018  Sotiris Papatheodorou  (sotirisp@protonmail.com)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# -*- coding: utf-8 -*-


import csv
import datetime
import sys
from .listitem import *


class LegoSet(ListItem):
    def __init__(self, item='0000-1', quantity=1):
        super().__init__()
        self.item = item
        self.quantity = quantity
        self.name = 'Set Name'
        self.year = datetime.date.today().year
        self.brickset_URI = 'https://brickset.com/sets/'+self.item
        self.rebrickable_URI = 'https://rebrickable.com/sets/'+self.item

    def __str__(self):
        s = [
            str(self.id),
            ') ',
            self.item,
            '\n  Name: ',
            self.name,
            '\n  Year: ',
            str(self.year),
            '\n'
        ]
        if self.tags:
            s.append('  Tags: ')
            [s.append(str(x)+', ') for x in self.tags]
            s.append('\b\b  \n')
        if self.brickset_URI:
            s.append('  Brickset URI: ' + str(self.brickset_URI) + '\n')
        if self.rebrickable_URI:
            s.append('  Rebrickable URI: ' + str(self.rebrickable_URI) + '\n')
        s.append('  Added on: ' + str(self.date_added) + '\n')
        return ''.join(s)


class LegoSets(ListItems):
    def __str__(self):
        s = [str(x)+'\n' for x in self.list]
        return ''.join(s)

    def read(self, fname):
        with open(fname, newline='') as f:
            reader = csv.reader(f)
            try:
                # Skip first line
                next(reader)
                for row in reader:
                    # Item and quantity
                    self.list.append(LegoSet(row[0], row[1]))
                    # ID
                    self.list[-1].id = len(self.list)
                    # Date added
                    self.list[-1].date_added = datetime.date.today()
            except csv.Error as e:
                sys.exit('file {}, line {}: {}'.format(filename,
                                                       reader.line_num, e))


